'use strict';
define([
    '../analytics',
    '../DropdownMenu',
    '../loyalty/Request',
    '../loyalty/OffersParser',
    '../links/cookieService',
    '../../../../lib/domQ'
], function (analytics, DropdownMenu, loyaltyService, offerParser, mnsCookieService, domQ) {

    /**
     * LoyaltyDropdown Constructor
     */
    function LoyaltyDropdown (el, options) {

        // set menu elements
        this.menuItem = el;
        this.options = options;
        this.menuItemLink = this.menuItem.querySelector('.navigation-header__sparks-link');
        this.menuItemDropdown = this.menuItem.querySelector('.navigation-dropdown__panel-wrapper');

        // set sparks login and hub page urls
        this.sparksLoginPageUrl = this.menuItemLink.getAttribute('data-sparks-login-link');
        this.sparksHubPageUrl = this.menuItemLink.getAttribute('data-sparks-hub-link');
    }

    LoyaltyDropdown.prototype = {

        /**
         * Initializes sparks dropdown if user is logged in and environment supports CORS
         */
        initialize: function () {

            if (this.envSupportCORS()) {
                // add sparks dropdown
                domQ(this.menuItem).addClass('dropdown');
                domQ(this.menuItemLink).addClass('navigation-dropdown__link navigation-dropdown__menuitem-link');
                this.addSparksDropdown();

                // toggle sparks links
                this.toggleSparksLinkUrl(this.menuItemLink);
                this.toggleSparksLinkUrl(this.menuItem.querySelector('#gotoSparksLink'));

                // bind sparks dropdown events
                this.bindOnMenuOpenEvent();
                this.preventMenuItemClick();
            }

            return this;
        },

        /**
         * Add sparks dropdown
         */
        addSparksDropdown: function() {
            DropdownMenu.get('HEADER_DROPDOWN_MENU').addMenuItemLink(this.menuItemLink, this.menuItemDropdown);
        },

        /**
         * Validate if environment has CORS support
         * @returns {boolean} TRUE if user is logged in and environment support CORS
         */
        envSupportCORS: function() {
            return loyaltyService.fullySupportsCORS();
        },

        /**
         * Toggle sparks links url for logged-in and sparks member user
         * @param {HTMLElement} DOM link element
         */
        toggleSparksLinkUrl: function(el) {
            var url = mnsCookieService.isUserSparksMember() ? el.getAttribute('data-sparks-hub-link') : el.getAttribute('data-sparks-login-link');
            el.setAttribute('href', url);
        },

        /**
         * Bind onMenuOpen event to load loyalty offers only once
         */
        bindOnMenuOpenEvent: function () {
            var self = this,
                removeOnMenuOpenListener;

            removeOnMenuOpenListener = DropdownMenu.get('HEADER_DROPDOWN_MENU').onMenuOpen(function(menuItem) {
                if (self.menuItemLink === menuItem) {
                    self.loadOffers();
                    removeOnMenuOpenListener();
                }
            });
        },

        /**
         * Initiate web request(s) to load loyalty offers
         */
        loadOffers: function() {
            loyaltyService.getAuthToken(this.options.authTokenUrl)
                .then(loyaltyService.getOffers.bind(loyaltyService, this.options.loyaltyOfferApi))
                .then(this.offersDataHandler.bind(this))
                .catch(this.removeSparksLoader.bind(this));
        },

        removeSparksLoader: function() {
            this.menuItemDropdown.querySelector('.navigation-sparks-dropdown__content').innerHTML = '';
        },

        /**
         * Loyalty offers web request handler
         * @param {Object} Offers data
         */
        offersDataHandler: function (offersData) {
            // Tibco down scenario
            if (offersData.numberOfOffers === 0 && offersData.availableSlots > 0) {
                return false;
            }

            this.renderOffersHtml(offerParser.parse(offersData));
        },

        /**
         * Render loyalty offers into dropdown
         * @param {Object} loyaltyOffers data
         */
        renderOffersHtml: function (loyaltyOffers) {
            var template = this.getContentTemplate();

            // set sparks user name
            domQ(template.querySelector('.navigation-sparks-dropdown__user'))
                .text(mnsCookieService.getUsername());

            // set sparks count
            domQ(template.querySelector('.navigation-sparks-dropdown__sparks-count'))
                .text(loyaltyOffers.customerSparks);

            // set offers html
            this.addActiveOffersToTemplate(template, loyaltyOffers.allActiveOffers);
            this.addAvailableSlottedOffers(template, loyaltyOffers.availableSlottedOffers);
            this.addAvailableBonusOffers(template, loyaltyOffers.availableBonusOffers);

            // set html content
            this.menuItemDropdown.querySelector('.navigation-sparks-dropdown__content').innerHTML = template.innerHTML;
        },

        /**
         * Prevents link redirection if dropdown is not open
         */
        preventMenuItemClick: function () {
            this.menuItemLink.addEventListener('click', this.preventDefaultMenuItemClick);
        },

        preventDefaultMenuItemClick: function (evt) {
            evt.preventDefault();
        },

        /**
         * Returns sparks dropdown template
         * @returns {HTMLElement}
         */
        getContentTemplate: function () {
            var template = document.createElement('div'),
                content = document.getElementById('sparksDropdownContentTemplate').innerHTML;

            template.innerHTML = content;
            return template.children[0];
        },

        /**
         * Returns sparks dropdown template
         * @returns {HTMLElement}
         */
        addActiveOffersToTemplate: function (template, offers) {
            var offerListEl = template.querySelector('.navigation-sparks-dropdown__active-offers'),
                offerListItemEl = offerListEl.removeChild(offerListEl.children[0]);

            // remove DOM list element if offers not available
            if (offers.length === 0) {
                offerListEl.style.display = 'none';
                return false;
            }

            offers.map(function(offerObj) {
                var listItem = offerListItemEl.cloneNode(true),
                    link = listItem.querySelector('a'),
                    img = listItem.querySelector('img'),
                    offerText = listItem.querySelector('span'),
                    offer = offerObj.offer,
                    href;

                // set offer link element
                domQ(link).attr('data-analytics-shop-online', offer.applicableOnline.toString())
                    .attr('data-analytics-shop-instore', offer.applicableInstore.toString());

                href = link.getAttribute('href');

                (offer.buyUrl && !offer.redeemed) ? domQ(link).attr('href', offer.buyUrl) : domQ(link).attr('href', href);

                // set offer image element
                domQ(img).attr('src', offer.imageUrl);

                // set offer text
                domQ(offerText).text(offer.offerName);

                // append offer list item
                offerListEl.appendChild(listItem);
            });
        },

        /**
         * Add available slotted to template
         */
        addAvailableSlottedOffers: function(template, offers) {
            var css = ['navigation-sparks-dropdown__selectable-offer--double', 'navigation-sparks-dropdown__selectable-offer--multiple'];

            if (offers.length === 0) {
                template.querySelector('.navigation-sparks-dropdown__avl-offers').style.display = 'none';
                return false;
            }

            domQ(template.querySelector('.navigation-sparks-dropdown__avl-offers a div'))
                .addClass(offers.length > 1 ? css.join[' '] : css[0]);

            domQ(template.querySelector('.navigation-sparks-dropdown__avl-offers .navigation-sparks-dropdown__notification'))
                .text(offers.length);
        },

        /**
         * Add available bonus to template
         */
        addAvailableBonusOffers: function(template, offers) {
            var css = ['navigation-sparks-dropdown__selectable-offer--double', 'navigation-sparks-dropdown__selectable-offer--multiple'];

            if (offers.length === 0) {
                template.querySelector('.navigation-sparks-dropdown__avl-bonus-offers').style.display = 'none';
                return false;
            }

            domQ(template.querySelector('.navigation-sparks-dropdown__avl-offers a div'))
                .addClass(offers.length > 1 ? css.join[' '] : css[0]);

            domQ(template.querySelector('.navigation-sparks-dropdown__avl-bonus-offers .navigation-sparks-dropdown__notification'))
                .text(offers.length);
        }
    };

    return LoyaltyDropdown;
});
