'use strict';
define([
    '../../../../lib/domQ',
    '../savedItems/savedItemService',
    '../carousel/Index'
], function (domQ, savedItemService, Carousel) {

    function SavedItems(el, options) {
        this.el = el;
        this.options = this.getOptions(options);
        this.savedItemViewCountEl = document.querySelector(this.options.savedItemViewCountEl);
        this.wrapper = document.querySelector(this.options.wrapper);
        this.carousel = null;
    }

    SavedItems.prototype = {

        /**
         * Returns saved items configuration options
         * @param {Object} options custom configuration options
         * @returns {Object} saved items configuration options
         */
        getOptions: function (options) {
            var defaultOptions = {
                wrapper: options.wrapper || '',
                savedItemViewCountEl: options.savedItemViewCountEl || ''
            };

            return Object.assign(defaultOptions, options);
        },

        /**
         * Initialize saved items component
         */
        init: function () {
            var generateSavedItemContent = this.generateSavedItemContent.bind(this);

            // subscibe saved item data change event
            savedItemService.onChange(function(data) {
                generateSavedItemContent(data);
            });

            // update saved item content
            this.updateSavedItemViewCount(savedItemService.itemCount);

            if (savedItemService.itemCount > 0) {
                savedItemService.loadData(this.options.savedItemCMD, this.options.requestParams);
            }

            // bind events
            this.bindEvents();
        },

        /**
         * Update saved item view count text
         * @params {Number} count saved item count
         */
        updateSavedItemViewCount: function(count) {
            this.savedItemViewCountEl.innerText = count;
        },

        /**
         * Generate HTML content for saved item
         * @param {Array<Object>} items Saved items list
         */
        generateSavedItemContent: function (items) {
            if (items.length > 0) {
                this.generateCarouselContent(items);
            }

            this.updateSavedItemViewCount(items.length);
            this.toggleSavedItemContent(items.length > 0);
        },

        /**
         * Generate carousel content for saved item
         * @param {Array<Object>} items Saved items list
         */
        generateCarouselContent: function(items) {
            this.wrapper.innerHTML = this.getSlides(items).outerHTML;

            new Carousel(this.el, {
                wrapper: '.navigation-hdr-carousel__content',
                slider: '.navigation-hdr-carousel__slides',
                slide: '.navigation-hdr-carousel__slides li',
                multiple: true,
                previousBtn: '.navigation-hdr-carousel__btn--previous',
                nextBtn: '.navigation-hdr-carousel__btn--next',
                type: 'horizontal',
                disableBtnCss: 'navigation-hdr-ctrl--link-disable'
            }).init();
        },

        /**
         * Toggles section to show/hide saved item content and not item message
         * @param {boolean} hasItems true if items available
         */
        toggleSavedItemContent: function(hasItems) {
            if (hasItems) {
                this.el.querySelector('.navigation-saved-items__empty').style.display = 'none';
                this.el.querySelector('.navigation-hdr-carousel--saved-items').style.display = 'block';
            } else {
                this.el.querySelector('.navigation-saved-items__empty').style.display = 'block';
                this.el.querySelector('.navigation-hdr-carousel--saved-items').style.display = 'none';
            }
        },

        /**
         * Returns saved items carousel slides
         * @param {Array<Object>} items saved items list
         * @returns {HTMLElement} saved items carousel slides
         */
        getSlides: function (items) {
            var slideContainer = domQ('ul').addClass('carousel__slides'),
                getSlide = this.getSlide.bind(this);

            items.map(function(item) {
                slideContainer.append(getSlide(item));
            });

            return slideContainer.el;
        },

        /**
         * Returns saved items carousel slide dom element
         * @param {Object} item saved item
         * @returns {HTMLElement} saved items carousel slides
         */
        getSlide: function(item) {
            return domQ('li')
                .append(this.getProductImageLink(item))
                .append(this.getProductDetails(item))
                .el;
        },

        /**
         * Returns saved items carousel slide product detail DOM
         * @param {Object} item saved item
         * @returns {HTMLElement} saved items carousel slides
         */
        getProductDetails: function (item) {
            return domQ('div').addClass('productInfo')
                .append(this.getProductAttributeHTML(item))
                .append(this.getPriceHTML(item))
                .append(this.getAddToBagBtn(item))
                .append(this.getRemoveBtn(item))
                .el;
        },

        /**
         * Returns saved item slide link element
         * @param {Object} item saved item
         * @returns {HTMLElement} saved item slide link element
         */
        getProductImageLink: function(item) {
            return domQ('a')
                .attr('href', item.productLink)
                .append(this.getProductImage(item))
                .el;
        },

        /**
         * Returns saved item slide item image
         * @param {Object} item saved item
         * @returns {HTMLElement} saved item slide item image
         */
        getProductImage: function(item) {
            return domQ('img')
                .attr('src', item.productImgUrl)
                .attr('alt', item.imgAltText)
                .el;
        },

        /**
         * Returns saved item slide item product attributes
         * @param {Object} item saved item
         * @returns {HTMLElement} saved item slide product attributes
         */
        getProductAttributeHTML: function(item) {
            return domQ('span').addClass('productAttributes')
                .text(item.productAttributes)
                .el;
        },

        /**
         * Returns saved item slide item add to bag button
         * @param {Object} item saved item
         * @returns {HTMLElement} saved item add to bag button
         */
        getAddToBagBtn: function(item) {
            // dont return add to bag button if item is out of stock
            if (item.outOfStock) {
                return null;
            }

            return domQ('button').addClass('btn btn--primary btn--xs')
                .attr('data-item-id', item.orderItemId)
                .text(item.addBagLink.linkText)
                .el;
        },

        /**
         * Returns saved item slide item remove button
         * @param {Object} item saved item
         * @returns {HTMLElement} saved item remove button
         */
        getRemoveBtn: function(item) {
            return domQ('button').addClass('btn btn--secondary btn--xs')
                .attr('data-item-id', item.orderItemId)
                .text(item.removeLink.linkText)
                .el;
        },

        /**
         * Returns item price dom element
         * @param {Object} item product item
         * @returns {Object} DOM Element
         */
        getPriceHTML: function(item) {
            return domQ('span').addClass('price')
                .html(item.itemPrice)
                .el;
        },

        /**
         * Returns Out of stock dom element
         * @returns {Object} DOM Element
         */
        getOutOfStockHTML: function() {
            return domQ('span').addClass('oos')
                .text('Out of stock')
                .el;
        },

        /**
         * Add to bag and remove button click event handler
         * @params {Object} Click Event
         */
        handleClick: function (evt) {
            var itemId = null;

            if (this.isRemoveBtnClicked(evt.target)) {
                itemId = evt.target.getAttribute('data-item-id');
                savedItemService.remove(savedItemService.getItemById(itemId), this.options.wcsBaseUrl);
            } else if (this.isAddBtnClicked(evt.target)) {
                itemId = evt.target.getAttribute('data-item-id');
                savedItemService.addToBag(savedItemService.getItemById(itemId), this.options.wcsBaseUrl);
            }
        },

        /**
         * Add click event handler to saved item container
         */
        bindEvents: function() {
            this.el.querySelector('.navigation-hdr-carousel--saved-items')
                .addEventListener('click', this.handleClick.bind(this));
        },

        /**
         * Check if target is remove buttom
         * @params {HTMLElement} Button
         * @returns {Boolean} TRUE if target is remove buttom
         */
        isRemoveBtnClicked: function(target) {
            return target.className.split(/\s+/).indexOf('btn--secondary') > -1;
        },

        /**
         * Check if target is add to bag buttom
         * @params {HTMLElement} Button
         * @returns {Boolean} TRUE if target is add to bag buttom
         */
        isAddBtnClicked: function(target) {
            return target.className.split(/\s+/).indexOf('btn--primary') > -1;
        }
    };

    return SavedItems;
});