'use strict';
define([
    'deep-equal',
    'xhr-promise',
    '../links/cookieService'
], function (deepEqual, XhrPromise, mnsCookieService) {

    var savedItemData = [],
        itemCount = null,
        changeListeners = [];

    // service API
    return {
        /**
         * Getter saved item count value
         * @returns {Number} Saved item count value
         */
        get itemCount() {
            if (itemCount === null) {
                itemCount = mnsCookieService.getSavedItemCountCookieValue();
            }

            return itemCount;
        },

        /**
         * Setter saved item count value
         * @params {Number} Saved item count value
         */
        set itemCount(value) {
            itemCount = value;
        },

        /**
         * Setter saved item count value
         * @params {Number} Saved item count value
         */
        onChange: function(callback) {
            if (changeListeners.indexOf(callback) > -1) {
                return;
            }

            // add change listener callback
            changeListeners.push(callback);

            // return remove listener function
            return function() {
                changeListeners = changeListeners.filter(function(item) {
                    return item !== callback;
                });
            };
        },

        /**
         * Returns saved item data
         * @returns {Object | null} saved item data
         */
        getData: function() {
            return savedItemData;
        },

        /**
         * Returns string parameter data from object
         * @returns {String} parameter data
         */
        toQueryString: function(paramsObj) {
            if(typeof paramsObj !== 'object') {
                return '';
            }

            var parts = [];
            for (var i in paramsObj) {
                if (paramsObj.hasOwnProperty(i)) {
                    parts.push(encodeURIComponent(i) + '=' + encodeURIComponent(paramsObj[i]));
                }
            }
            return parts.join('&');
        },

        /**
         * Load saved item data
         */
        loadData: function(savedItemCMD, requestParams) {
            var updateData = this.update.bind(this);
            var toQueryString = this.toQueryString.bind(this);

            var params = Object.assign({resJSON: true}, requestParams);

            return this.request({
                url: savedItemCMD,
                method: 'POST',
                data: toQueryString(params)
            }).then(function (response) {
                if (response.status === 200) {
                    return updateData(response.responseText);
                }
            }).catch(function(){
                //Handle errors here
            });
        },

        /**
         * Remove saved item
         * @params {Object} Saved item object
         */
        remove: function(removeItem, wcsBaseUrl) {
            var self = this;

            return this.request({
                url: wcsBaseUrl + removeItem.removeLink.action,
                method: 'POST',
                data: self.toQueryString(removeItem.removeLink.params)
            }).then(function(response) {
                if (response.status === 200) {
                    return self.update(savedItemData.filter(function(item) {
                        var opts = {strict: true};
                        return !deepEqual(item, removeItem, opts);
                        //return item !== removeItem;
                    }));
                }
            }).catch(function(){
                //Handle errors here
            });
        },

        /**
         * Add saved item to the basket
         * @params {Object} Saved item object
         */
        addToBag: function(addItem, wcsBaseUrl) {
            var self = this;

            return this.request({
                url: wcsBaseUrl + addItem.addBagLink.action,
                method: 'POST',
                data: self.toQueryString(addItem.addBagLink.params)
            }).then(function(response) {
                if (response.status === 200) {
                    return self.update(savedItemData.filter(function(item) {
                        var opts = {strict: true};
                        return !deepEqual(item, addItem, opts);
                        //return item !== addItem;
                    }));
                }
            }).catch(function(){
                //Handle errors here
            });
        },

        /**
         * Update saved item data
         * @params {Object} Saved items data
         */
        update: function(data) {
            savedItemData = data;

            changeListeners.map(function(callback) {
                callback(savedItemData);
            });
        },

        /**
         * Returns saved item
         * @params {srting} Saved item id
         * @returns {Object} Saved item
         */
        getItemById: function(id) {
            return savedItemData.filter(function(item) {
                return item.orderItemId === id;
            })[0] || null;
        },

        /**
         * xhr-promise request promise
         */
        request: function(options) {
            var xhrp = new XhrPromise();
            return xhrp.send(options);
        }
    };
});
