import carouselHelper from './helper/carouselHelper';
import Carousel from '../../js/carousel/Index';
import jsdom from 'jsdom';

describe('Progressive header Carousel component', function() {
    var window,
        horizontalCarouselDOM,
        verticalCarouselDOM,
        carousel;

    beforeEach(function() {
        var options = carouselHelper.getDefaultOptions();

        window = carouselHelper.getCarouselWindow(jsdom);
        horizontalCarouselDOM = window.document.querySelector('#horizontalCarousel');
        verticalCarouselDOM = window.document.querySelector('#verticalCarousel');

        // mock offset width of elements as jsdom not supporting cssom
        Object.defineProperty(HTMLElement.prototype, 'offsetWidth', {
            writable: true,
            configurable: true
        });
        Object.defineProperty(HTMLElement.prototype, 'offsetHeight', {
            writable: true,
            configurable: true
        });

        horizontalCarouselDOM.querySelector(options.wrapper).offsetWidth = 400;
        horizontalCarouselDOM.querySelector(options.slide).offsetWidth = 200;
        // mock offset width of elements as jsdom not supporting cssom
        verticalCarouselDOM.querySelector(options.wrapper).offsetHeight = 100;
        verticalCarouselDOM.querySelector(options.slide).offsetHeight = 100;

        carousel = new Carousel(horizontalCarouselDOM);
    });

    afterEach(function(){
        window.close();
        Object.defineProperty(HTMLElement.prototype, 'offsetWidth', {
            writable: false
        });
        Object.defineProperty(HTMLElement.prototype, 'offsetHeight', {
            writable: false
        });
    });

    describe('carousel types', function() {
        it('should have horizontal and vertical carousel types config', function() {
            var types = {
                HORIZONTAL: 'horizontal',
                VERTICAL: 'vertical'
            };

            expect(Carousel.TYPES).to.deep.equal(types);
        });
    });

    describe('constructor method', function() {
        it('should return Carousel instance', function() {
            expect(carousel).to.be.instanceof(Carousel);
        });
    });

    describe('getOptions method', function() {
        var options = carouselHelper.getDefaultOptions();

        it('should return default carousel options', function() {
            expect(carousel.options).to.deep.equal(carouselHelper.getDefaultOptions());
        });

        it('should return custom carousel options', function() {
            var options = carouselHelper.getCustomOptions();

            carousel = new Carousel(horizontalCarouselDOM, options);
            expect(carousel.options).to.deep.equal(options);
        });
    });

    describe('init method', function() {
        var options;

        beforeEach(function() {
            options = carouselHelper.getDefaultOptions();
        });

        it('should set correct settings for horizontal carousel', function() {
            carousel.init();

            expect(carousel.wrapper).to.be.equal(horizontalCarouselDOM.querySelector(options.wrapper));
            expect(carousel.wrapperSize).to.be.equal(parseInt(carousel.wrapper.style.width));
            expect(carousel.slider).to.be.equal(horizontalCarouselDOM.querySelector(options.slider));
            expect(carousel.nextBtn).to.be.equal(horizontalCarouselDOM.querySelector(options.nextBtn));
            expect(carousel.previousBtn).to.be.equal(horizontalCarouselDOM.querySelector(options.previousBtn));
            expect(carousel.slideIndex).to.be.equal(0);
            expect(carousel.totalSlides).to.be.equal(horizontalCarouselDOM.querySelectorAll(options.slide).length);
            expect(carousel.slideSize).to.be.equal(parseInt(horizontalCarouselDOM.querySelector(options.slide).style.width));
            expect(carousel.slidesinView).to.be.equal(2);
        });

        it('should set correct settings for vertical carousel', function() {
            options.type = Carousel.TYPES.VERTICAL;
            carousel = new Carousel(verticalCarouselDOM, options);
            carousel.init();

            expect(carousel.wrapper).to.be.equal(verticalCarouselDOM.querySelector(options.wrapper));
            expect(carousel.wrapperSize).to.be.equal(parseInt(carousel.wrapper.style.width));
            expect(carousel.slider).to.be.equal(verticalCarouselDOM.querySelector(options.slider));
            expect(carousel.nextBtn).to.be.equal(verticalCarouselDOM.querySelector(options.nextBtn));
            expect(carousel.previousBtn).to.be.equal(verticalCarouselDOM.querySelector(options.previousBtn));
            expect(carousel.slideIndex).to.be.equal(0);
            expect(carousel.totalSlides).to.be.equal(verticalCarouselDOM.querySelectorAll(options.slide).length);
            expect(carousel.slideSize).to.be.equal(parseInt(verticalCarouselDOM.querySelector(options.slide).style.width));
            expect(carousel.slidesinView).to.be.equal(1);
        });

        it('should call bindEvents method once to bind carousel events', function() {
            var spy = sinon.spy(carousel, 'bindEvents');

            carousel.init();

            expect(spy.calledOnce).to.be.ok;
        });

        it('should call toggleNavBtnState method once to set initial next / previous buttons state', function() {
            var spy = sinon.spy(carousel, 'toggleNavBtnState');

            carousel.init();

            expect(spy.calledOnce).to.be.ok;
        });
    });

    describe('toggleNavBtnState method', function() {
        var spyDisablePreviousBtn,
            spyEnablePreviousBtn,
            spyDisableNextBtn,
            spyEnableNextBtn;

        beforeEach(function() {
            carousel.init();
            spyDisablePreviousBtn = sinon.spy(carousel, 'disablePreviousBtn');
            spyEnablePreviousBtn = sinon.spy(carousel, 'enablePreviousBtn');
            spyDisableNextBtn = sinon.spy(carousel, 'disableNextBtn');
            spyEnableNextBtn = sinon.spy(carousel, 'enableNextBtn');
        });

        it('should disable previous link state if current slide is first slide in the carousel', function() {
            carousel.isFirstSlide = sinon.stub();
            carousel.isFirstSlide.returns(true);
            carousel.toggleNavBtnState();

            expect(carousel.isFirstSlide.calledOnce).to.be.ok;
            expect(spyDisablePreviousBtn.calledOnce).to.be.ok;
            expect(spyEnablePreviousBtn.called).to.not.be.ok;
        });

        it('should enable previous link state if current slide is not first slide in the carousel', function() {
            carousel.isFirstSlide = sinon.stub();
            carousel.isFirstSlide.returns(false);
            carousel.toggleNavBtnState();

            expect(carousel.isFirstSlide.calledOnce).to.be.ok;
            expect(spyDisablePreviousBtn.called).to.not.be.ok;
            expect(spyEnablePreviousBtn.calledOnce).to.be.ok;
        });

        it('should disable next link state if current slide is last slide in the carousel', function() {
            carousel.isLastSlide = sinon.stub();
            carousel.isLastSlide.returns(true);
            carousel.toggleNavBtnState();

            expect(carousel.isLastSlide.calledOnce).to.be.ok;
            expect(spyDisableNextBtn.calledOnce).to.be.ok;
            expect(spyEnableNextBtn.called).to.not.be.ok;
        });

        it('should enable next link state if current slide is not last slide in the carousel', function() {
            carousel.isLastSlide = sinon.stub();
            carousel.isLastSlide.returns(false);
            carousel.toggleNavBtnState();

            expect(carousel.isLastSlide.calledOnce).to.be.ok;
            expect(spyDisableNextBtn.called).to.not.be.ok;
            expect(spyEnableNextBtn.calledOnce).to.be.ok;
        });
    });

    describe('disableNextBtn method', function() {
        it('should disable next slide link', function() {
            carousel.init();
            carousel.disableNextBtn();

            expect(carousel.nextBtn.className).to.contain(carouselHelper.getDefaultOptions().disableBtnCss);
        });
    });

    describe('enableNextBtn method', function() {
        it('should enable next slide link', function() {
            carousel.init();
            carousel.disableNextBtn();
            carousel.enableNextBtn();

            expect(carousel.nextBtn.className).to.not.contain(carouselHelper.getDefaultOptions().disableBtnCss);
        });
    });

    describe('disablePreviousBtn method', function() {
        it('should disable previous slide link', function() {
            carousel.init();
            carousel.disablePreviousBtn();

            expect(carousel.previousBtn.className).to.contain(carouselHelper.getDefaultOptions().disableBtnCss);
        });
    });

    describe('enablePreviousBtn method', function() {
        it('should enable previous slide link', function() {
            carousel.init();
            carousel.disablePreviousBtn();
            carousel.enablePreviousBtn();

            expect(carousel.previousBtn.className).to.not.contain(carouselHelper.getDefaultOptions().disableBtnCss);
        });
    });

    describe('slideNext method', function() {

        var spyToggleLinkState,
            spyAnimate,
            event;

        beforeEach(function() {
            carousel.init();
            spyToggleLinkState = sinon.spy(carousel, 'toggleNavBtnState');
            spyAnimate = sinon.spy(carousel, 'animate');
            event = {
                preventDefault: sinon.spy(),
                stopPropagation: sinon.spy(),
                target: document.createElement('div')
            };
        });

        it('should call preventDefault and stopPropagation to prevent default click and bubbling up', function() {
            carousel.slideNext(event); 
            expect(event.preventDefault).to.have.been.called;
            expect(event.stopPropagation).to.have.been.called;
        });

        it('should be able to move to next slide', function() {
            var status;

            status = carousel.slideNext(event);            
            expect(status).to.be.equal(true);
            expect(carousel.slideIndex).to.be.equal(1);
            expect(spyToggleLinkState.calledOnce).to.be.ok;
            expect(spyAnimate.calledOnce).to.be.ok;
        });

        it('should not be able to move to next slide if current slide is last slide', function() {
            var status;

            carousel.slideNext(event);
            carousel.slideNext(event);
            carousel.slideNext(event);
            status = carousel.slideNext(event);

            expect(status).to.be.equal(false);
            expect(carousel.slideIndex).to.be.equal(2);
            expect(spyToggleLinkState.calledTwice).to.be.ok;
            expect(spyAnimate.calledTwice).to.be.ok;
        });
    });

    describe('slidePrevious method', function() {
        var spyToggleLinkState,
            spyAnimate,
            event;

        beforeEach(function() {
            carousel.init();
            spyToggleLinkState = sinon.spy(carousel, 'toggleNavBtnState');
            spyAnimate = sinon.spy(carousel, 'animate');
            event = {
                preventDefault: sinon.spy(),
                stopPropagation: sinon.spy(),
                target: document.createElement('div')
            };
        });

        it('should call preventDefault and stopPropagation to prevent default click and bubbling up', function() {
            carousel.slidePrevious(event); 
            expect(event.preventDefault).to.have.been.called;
            expect(event.stopPropagation).to.have.been.called;
        });

        it('should not be able to move to previous slide on first time', function() {
            var status;

            status = carousel.slidePrevious(event);
            expect(status).to.be.equal(false);
            expect(carousel.slideIndex).to.be.equal(0);
            expect(spyToggleLinkState.called).to.not.be.ok;
            expect(spyAnimate.called).to.not.be.ok;
        });

        it('should be able to move to previous slide', function() {
            var status;

            carousel.slideNext(event);
            status = carousel.slidePrevious(event);
            expect(status).to.be.equal(true);
            expect(carousel.slideIndex).to.be.equal(0);
            expect(spyToggleLinkState.calledTwice).to.be.ok;
            expect(spyAnimate.calledTwice).to.be.ok;
        });

        it('should not be able to move to previous slide if current slide is the first slide', function() {
            var status;

            carousel.slideNext(event);
            carousel.slidePrevious(event);
            status = carousel.slidePrevious(event);

            expect(status).to.be.equal(false);
            expect(carousel.slideIndex).to.be.equal(0);
            expect(spyToggleLinkState.calledTwice).to.be.ok;
            expect(spyAnimate.calledTwice).to.be.ok;
        });
    });

    describe('isFirstSlide method', function() {
        it('should return true if current slide is first slide of carousel', function() {
            carousel.init();
            carousel.slideIndex = 0;

            expect(carousel.isFirstSlide()).to.ok;
        });

        it('should return false if current slide is not first slide of carousel', function() {
            carousel.init();
            carousel.slideIndex = 1;

            expect(carousel.isFirstSlide()).to.not.be.ok;
        });
    });

    describe('isLastSlide method', function() {
        describe('carousel with single slide move', function() {
            it('should return true if current slide is last slide of carousel', function() {
                var options = carouselHelper.getDefaultOptions();

                options.multiple = false;
                carousel = new Carousel(horizontalCarouselDOM, options);
                carousel.init();
                carousel.slideIndex = carousel.totalSlides - 1;

                expect(carousel.isLastSlide()).to.be.ok;
            });

            it('should return false if current slide is not last slide of carousel', function() {
                var options = carouselHelper.getDefaultOptions();

                options.multiple = false;
                carousel = new Carousel(horizontalCarouselDOM, options);
                carousel.init();
                carousel.slideIndex = carousel.totalSlides - 2;

                expect(carousel.isLastSlide()).to.not.be.ok;
            });
        });

        describe('carousel with multiple slide move', function() {
            it('should return true if current slide is last slide of carousel', function() {
                var options = carouselHelper.getDefaultOptions();

                options.multiple = true;
                carousel = new Carousel(horizontalCarouselDOM, options);
                carousel.init();
                carousel.slideIndex = Math.ceil(carousel.totalSlides / carousel.slidesinView) - 1;

                expect(carousel.isLastSlide()).to.be.ok;
            });

            it('should return false if current slide is not last slide of carousel', function() {
                var options = carouselHelper.getDefaultOptions();

                options.multiple = true;
                carousel = new Carousel(horizontalCarouselDOM, options);
                carousel.init();
                carousel.slideIndex = Math.floor(carousel.totalSlides / carousel.slidesinView) - 2;

                expect(carousel.isLastSlide()).to.not.be.ok;
            });
        });
    });

    describe('getPostion method', function() {

        var event;

        beforeEach(function() {
            event = {
                preventDefault: sinon.spy(),
                stopPropagation: sinon.spy(),
                target: document.createElement('div')
            };
        });

        it('should return slider position value for carousel with single slide move', function() {
            var options = carouselHelper.getDefaultOptions(),
                position;

            options.multiple = false;
            carousel = new Carousel(horizontalCarouselDOM, options);
            carousel.init();
            carousel.slideNext(event);
            position = -(carousel.slideSize * carousel.slideIndex);

            expect(carousel.getPosition()).to.be.equal(position);
        });

        it('should return slider position value for carousel with multiple slide move', function() {
            var options = carouselHelper.getDefaultOptions(),
                position;

            options.multiple = true;
            carousel = new Carousel(horizontalCarouselDOM, options);
            carousel.init();
            carousel.slideNext(event);
            position = -(carousel.slideSize * carousel.slideIndex * carousel.slidesinView);

            expect(carousel.getPosition()).to.be.equal(position);
        });
    });
});