import DockOverlay from '../../js/dockOverlay/Index';
import helper from './helper/dockOverlayHelper';

describe('Progressive Header DockOverlay Component', function () {

    var overlayDOM,
        targetDOM,
        overlay;

    beforeEach(function () {
        overlayDOM = helper.getOverlayDOM();
        targetDOM = helper.getOverlayTarget();
        overlay = new DockOverlay(overlayDOM, targetDOM);

        document.body.appendChild(overlayDOM);
        document.body.appendChild(targetDOM);
    });

    afterEach(function () {
        overlayDOM.remove();
        targetDOM.remove();
    });

    describe('DockOverlay.SHOW_HIDE_DELAY', function () {
        it('should have delay configuration on show/hide overlay', function () {
            expect(DockOverlay.SHOW_HIDE_DELAY).to.be.equal(100);
        });
    });

    describe('constructor method', function () {

        var bindEventsSpy;

        beforeEach(function () {
            bindEventsSpy = sinon.spy(DockOverlay.prototype, 'bindEvents');
        });

        afterEach(function () {
            DockOverlay.prototype.bindEvents.restore();
        });

        it('should return DockOverlay instance', function () {
            expect(overlay).to.be.instanceof(DockOverlay);
            expect(overlay.element).to.be.equal(overlayDOM);
            expect(overlay.target).to.be.equal(targetDOM);
            expect(overlay.onInitShowCallbacks).to.deep.equal([]);
            expect(overlay.onShowCallbacks).to.deep.equal([]);
        });

        it('should call bindEvents method', function () {
            overlay = new DockOverlay(overlayDOM, targetDOM);
            expect(bindEventsSpy.calledOnce).to.be.ok;
        });
    });

    describe('getOptions method', function () {
        it('should return default dock overlay options', function () {
            expect(overlay.options).to.deep.equal(helper.getDefaultOptions());
        });

        it('should return custom dock overlay options', function () {
            overlay = new DockOverlay(overlayDOM, targetDOM, helper.getCustomOptions());
            ////////////
            expect(overlay.options).to.deep.equal(helper.getCustomOptions());
        });
    });

    describe('executeOnInitShowCallbacks method', function () {
        it('should execute onInitShowCallbacks list', function () {
            overlay.onInitShowCallbacks = [sinon.stub(), sinon.stub(), sinon.stub()];
            overlay.executeOnInitShowCallbacks();

            expect(overlay.onInitShowCallbacks[0].calledOnce).to.be.ok;
            expect(overlay.onInitShowCallbacks[1].calledOnce).to.be.ok;
            expect(overlay.onInitShowCallbacks[2].calledOnce).to.be.ok;
        });
    });

    describe('content method', function () {
        it('should insert content into doc overlay content wrapper', function () {
            overlay.content('<p>Overlay Content</p>');
            expect(overlay.element.querySelector(overlay.options.contentWrapper).innerHTML).to.be.equal('<p>Overlay Content</p>');
        });
    });

    describe('show method', function () {
        it('should show overlay', function () {
            var spyExecuteOnShowCallbacks = sinon.spy(overlay, 'executeOnShowCallbacks'),
                clock = sinon.useFakeTimers();

            overlay.position = sinon.spy();
            overlay.show();
            clock.tick(3000);
            if (overlay.element.firstElementChild && overlay.element.firstElementChild.firstElementChild) {

                expect(spyExecuteOnShowCallbacks.calledOnce).to.equal(true);
                expect(overlay.position.calledOnce).to.equal(true);

                expect(overlay.element.className).to.contain(overlay.options.displayCss);
                expect(overlay.element.className).to.contain(overlay.options.transitionCss);
            }
            else {
                expect(overlay.element.className).to.not.contain(overlay.options.displayCss);
                expect(overlay.element.className).to.not.contain(overlay.options.transitionCss);
            }
            clock.restore();
        });
    });

    describe('hide method', function () {
        it('should hide overlay', function () {
            var clock = sinon.useFakeTimers();

            overlay.position = sinon.stub();
            overlay.show();
            overlay.hide();
            clock.tick(3000);
            expect(overlay.element.className).to.not.contain(overlay.options.displayCss);
            expect(overlay.element.className).to.not.contain(overlay.options.transitionCss);
            clock.restore();
        });
    });

    describe('stick method', function () {
        it('should hide overlay if hovered after SHOW_HIDE_DELAY', function () {
            overlay.hideOverlayTimeout = 0;

            expect(overlay.stick()).to.not.be.ok;
            expect(overlay.element.className).to.not.contain(overlay.options.displayCss);
            expect(overlay.element.className).to.not.contain(overlay.options.transitionCss);
        });

        it('should show overlay if hovered before SHOW_HIDE_DELAY', function () {
            var clock = sinon.useFakeTimers();

            overlay.hideOverlayTimeout = clock.tick(3000);

            expect(overlay.stick()).to.be.ok;
            expect(overlay.element.className).to.contain(overlay.options.transitionCss);

            clock.restore();
        });
    });

    describe('enable/disable method', function () {

        it('should disable overlay', function () {
            overlay.disable();

            expect(overlay.element.className).to.contain(overlay.options.disableCss);
        });

        it('should enable overlay', function () {
            overlay.enable();

            expect(overlay.element.className).to.not.contain(overlay.options.disableCss);
        });
    });

    describe('executeOnShowCallbacks method', function () {
        it('should execute onShowCallbacks list', function () {
            overlay.onShowCallbacks = [sinon.spy(), sinon.spy(), sinon.spy()];
            overlay.executeOnShowCallbacks();

            expect(overlay.onShowCallbacks[0].calledOnce).to.be.ok;
            expect(overlay.onShowCallbacks[1].calledOnce).to.be.ok;
            expect(overlay.onShowCallbacks[2].calledOnce).to.be.ok;
        });
    });

    describe('onInitShowEvent method', function () {
        var func1, func2;

        beforeEach(function () {
            func1 = sinon.spy();
            func2 = sinon.spy();
        });

        it('should add callback to onInitShowCallbacks list', function () {
            overlay.onInitShowEvent(func1);
            overlay.onInitShowEvent(func2);

            expect(overlay.onInitShowCallbacks[0]).to.be.equal(func1);
            expect(overlay.onInitShowCallbacks[1]).to.be.equal(func2);
        });

        it('should remove callback from onInitShowCallbacks list', function () {
            overlay.onInitShowEvent(func1)();
            overlay.onInitShowEvent(func2)();

            expect(overlay.onInitShowCallbacks.length).to.be.equal(0);
        });
    });

    describe('onShowEvent method', function () {
        var func1, func2;

        beforeEach(function () {
            func1 = sinon.spy();
            func2 = sinon.spy();
        });

        it('should add callback to onShowCallbacks list', function () {
            overlay.onShowEvent(func1);
            overlay.onShowEvent(func2);

            expect(overlay.onShowCallbacks[0]).to.be.equal(func1);
            expect(overlay.onShowCallbacks[1]).to.be.equal(func2);
        });

        it('should remove callback from onShowCallbacks list', function () {
            overlay.onShowEvent(func1)();
            overlay.onShowEvent(func2)();

            expect(overlay.onShowCallbacks.length).to.be.equal(0);
        });
    });

    describe('position method', function () {

        beforeEach(function () {
            overlay.calcPositionX = sinon.stub().returns(100);
            overlay.calcPositionY = sinon.stub().returns(200);
        });

        it('should set dock overlay element top and left position', function () {
            overlay.position();
            expect(overlay.element.style.top).to.be.equal(200 + 'px');
        });
    });

    describe('calcPositionX method', function () {

        beforeEach(function () {
            targetDOM.setAttribute('style', 'position:absolute;left:0;top:0;width:100px;height:50px');
        });

        it('should align dock overlay left to the target element', function () {
            var xPos = 0;

            overlay.options.align = 'LEFT';
            overlay.calcPositionX();
            expect(overlay.calcPositionX()).to.be.equal(xPos);
        });

        it('should align dock overlay right to the target element', function () {
            var xPos = -100;

            overlay.options.align = 'RIGHT';
            overlay.calcPositionX();
            expect(overlay.calcPositionX()).to.be.equal(xPos);
        });

        it('should align dock overlay center to the target element', function () {
            var xPos = 0;

            overlay.calcPositionX();
            expect(overlay.calcPositionX()).to.be.equal(xPos);
        });
    });

    describe('calcPositionY method', function () {
        beforeEach(function () {
            targetDOM.setAttribute('style', 'position:absolute;left:0;top:0;width:100px;height:50px');
        });

        it('should position dock overlay top to the target element', function () {
            overlay.options.direction = 'DOWN';
            expect(overlay.calcPositionY()).to.be.equal(50);
        });

        it('should align dock overlay down to the target element', function () {
            overlay.options.direction = 'UP';
            expect(overlay.calcPositionY()).to.be.equal(-200);
        });
    });
});