import LoyaltyDropdown from '../../js/loyalty/Index';
import loyaltyService from '../../js/loyalty/Request';
import mnsCookieService from '../../js/links/cookieService';
import DropdownMenu from '../../js/DropdownMenu';
import offersParser from '../../js/loyalty/OffersParser';
import Helper from './helper/helper';
import jsdom from 'jsdom';

var loyaltyDOM,
    window;

describe('Progressive Header Sparks drop down', function() {
    var dropdownContractorOptions = {
        authTokenUrl: '',
        loyaltyOfferApi: ''
    };
    beforeEach(function() {
        window = Helper.getContentWindow(jsdom);
        loyaltyDOM = window.document.documentElement.querySelector('#loyaltyDropdownWrap');
    });

    afterEach(function() {
        window.close();
    });

    describe('Loyalty dropdown constructor', function() {

        it('should return an instance of LoyaltyDropdown', function() {
            expect(getDropdownInstance(dropdownContractorOptions)).to.be.instanceof(LoyaltyDropdown);
        });

        it('should set instance properties', function() {
            var dropdown = getDropdownInstance(dropdownContractorOptions);

            expect(dropdown.menuItem).to.be.equal(loyaltyDOM.querySelector('li'));
            expect(dropdown.menuItemLink).to.be.equal(loyaltyDOM.querySelector('li a'));
            expect(dropdown.menuItemDropdown).to.be.equal(loyaltyDOM.querySelector('li div'));
            expect(dropdown.sparksLoginPageUrl).to.be.equal(loyaltyDOM.querySelector('li a').getAttribute('data-sparks-login-link'));
            expect(dropdown.sparksHubPageUrl).to.be.equal(loyaltyDOM.querySelector('li a').getAttribute('data-sparks-hub-link'));
        });
    });

    describe('initSparksDropdown method', function() {
        beforeEach(function() {
            sinon.stub(LoyaltyDropdown.prototype, 'envSupportCORS').returns(true);
            sinon.stub(LoyaltyDropdown.prototype, 'addSparksDropdown');
            sinon.stub(LoyaltyDropdown.prototype, 'toggleSparksLinkUrl');
            sinon.stub(LoyaltyDropdown.prototype, 'bindOnMenuOpenEvent');
            sinon.stub(LoyaltyDropdown.prototype, 'preventMenuItemClick');
            getDropdownInstance(dropdownContractorOptions).initialize();
        });

        afterEach(function() {
            LoyaltyDropdown.prototype.addSparksDropdown.restore();
            LoyaltyDropdown.prototype.toggleSparksLinkUrl.restore();
            LoyaltyDropdown.prototype.bindOnMenuOpenEvent.restore();
            LoyaltyDropdown.prototype.preventMenuItemClick.restore();
            LoyaltyDropdown.prototype.envSupportCORS.restore();
        });

        it('should initialize loyalty dropdown if user is logged in or environment support CORS', function() {
            expect(LoyaltyDropdown.prototype.addSparksDropdown.calledOnce).to.be.ok;
            expect(LoyaltyDropdown.prototype.toggleSparksLinkUrl.calledTwice).to.be.ok;
            expect(LoyaltyDropdown.prototype.bindOnMenuOpenEvent.calledOnce).to.be.ok;
            expect(LoyaltyDropdown.prototype.preventMenuItemClick.calledOnce).to.be.ok;
        });

        it('should have dropdown class added if a user is logged in and a sparks member', function() {
            expect(loyaltyDOM.querySelector('li').className.trim()).to.equal('dropdown');
        });
        it('should have dropdown__menuitem-link class added if a user is logged in and a sparks member', function() {
            expect(loyaltyDOM.querySelector('a').className.trim()).to.contain('navigation-dropdown__menuitem-link');
            expect(loyaltyDOM.querySelector('a').className.trim()).to.contain('navigation-dropdown__link');
        });
    });

    describe('addSparksDropdown method', function() {

        beforeEach(function() {
            sinon.stub(DropdownMenu, 'get');
        });

        afterEach(function() {
            DropdownMenu.get.restore();
        });

        it('should add sparks dropdown menu to header dopdown', function() {
            var addMenuItemLinkFunc = sinon.stub();

            DropdownMenu.get.returns({
                addMenuItemLink: addMenuItemLinkFunc
            });
            getDropdownInstance(dropdownContractorOptions).addSparksDropdown();
            expect(addMenuItemLinkFunc.getCall(0).args[0]).to.be.equal(loyaltyDOM.querySelector('a'));
            expect(addMenuItemLinkFunc.getCall(0).args[1]).to.be.equal(loyaltyDOM.querySelector('div'));
        });
    });

    describe('envSupportCORS method', function() {
        it('should return TRUE if user is logged in and environment supports CORS', function() {

            sinon.stub(mnsCookieService, 'isUserLoggedIn').returns(true);
            sinon.stub(loyaltyService, 'fullySupportsCORS').returns(true);

            expect(getDropdownInstance(dropdownContractorOptions).envSupportCORS).to.be.ok;

            // restore
            mnsCookieService.isUserLoggedIn.restore();
            loyaltyService.fullySupportsCORS.restore();
        });

        it('should return FALSE if user is not logged in and environment does not supports CORS', function() {

            sinon.stub(mnsCookieService, 'isUserLoggedIn').returns(false);
            sinon.stub(loyaltyService, 'fullySupportsCORS').returns(false);

            expect(getDropdownInstance(dropdownContractorOptions).envSupportCORS).to.be.ok;

            // restore
            mnsCookieService.isUserLoggedIn.restore();
            loyaltyService.fullySupportsCORS.restore();
        });
    });

    describe('toggleSparksLinkUrl method', function() {
        var mockedIsUserSparksMember;

        beforeEach(function() {
            mockedIsUserSparksMember = sinon.stub(mnsCookieService, 'isUserSparksMember');
        });

        afterEach(function() {
            mockedIsUserSparksMember.restore();
        });

        it('should add loyalty hub link if user is a spark member', function() {
            mockedIsUserSparksMember.returns(true);
            getDropdownInstance(dropdownContractorOptions).toggleSparksLinkUrl(loyaltyDOM.querySelector('a'));
            expect(loyaltyDOM.querySelector('a').getAttribute('href')).to.be.equal('SPARKS_HUB_URL');

        });

        it('should add loyalty login link if user is not a spark member', function() {
            mockedIsUserSparksMember.returns(false);
            getDropdownInstance(dropdownContractorOptions).toggleSparksLinkUrl(loyaltyDOM.querySelector('a'));
            expect(loyaltyDOM.querySelector('a').getAttribute('href')).to.be.equal('SPARKS_LOGIN_URL');
        });
    });

    describe('loadOffers method', function() {

        var mockGetAuthToken,
            mockGetOffers;

        beforeEach(function() {
            mockGetAuthToken = sinon.stub(loyaltyService, 'getAuthToken'),
            mockGetOffers = sinon.stub(loyaltyService, 'getOffers');
        });

        afterEach(function() {
            loyaltyService.getAuthToken.restore();
            loyaltyService.getOffers.restore();
            LoyaltyDropdown.prototype.offersDataHandler.restore();
        });

        it('should load and parse offers if auth token available', function(done) {
            var offers = {},
                authTokenPromise = Promise.resolve('AUTH_TOKEN'),
                offersPromise = Promise.resolve(offers);

            mockGetAuthToken.returns(authTokenPromise);
            mockGetOffers.returns(offersPromise);
            sinon.stub(LoyaltyDropdown.prototype, 'offersDataHandler');

            getDropdownInstance(dropdownContractorOptions).loadOffers();

            setTimeout(function() {
                expect(LoyaltyDropdown.prototype.offersDataHandler.calledOnce).to.be.ok;
                done();
            }, 0);
        });
    });

    describe('offersDataHandler method', function() {
        it('should not attempt to render the slots if no offers available', function() {
            var offersData = getDropdownInstance(dropdownContractorOptions).offersDataHandler({
                numberOfOffers: 0,
                availableSlots: 1
            });

            expect(offersData).to.be.false;
        });

        it('should render loyalty dropdown html', function() {
            var parsedData = {};

            sinon.stub(offersParser, 'parse').returns(parsedData);
            sinon.stub(LoyaltyDropdown.prototype, 'renderOffersHtml');
            getDropdownInstance(dropdownContractorOptions).offersDataHandler({
                numberOfOffers: 1,
                availableSlots: 1
            });

            expect(LoyaltyDropdown.prototype.renderOffersHtml.calledWith(parsedData)).to.be.ok;

            // restore
            offersParser.parse.restore();
            LoyaltyDropdown.prototype.renderOffersHtml.restore();
        });
    });

    describe('renderOffersHtml method', function() {

        var mockGetContentTemplate;

        beforeEach(function() {
            sinon.stub(mnsCookieService, 'getUsername').returns('SparksUser');
            sinon.stub(LoyaltyDropdown.prototype, 'addActiveOffersToTemplate');
            sinon.stub(LoyaltyDropdown.prototype, 'addAvailableSlottedOffers');
            sinon.stub(LoyaltyDropdown.prototype, 'addAvailableBonusOffers');
            mockGetContentTemplate = sinon.stub(LoyaltyDropdown.prototype, 'getContentTemplate');
        });

        afterEach(function() {
            mnsCookieService.getUsername.restore();
            LoyaltyDropdown.prototype.addActiveOffersToTemplate.restore();
            LoyaltyDropdown.prototype.addAvailableSlottedOffers.restore();
            LoyaltyDropdown.prototype.addAvailableBonusOffers.restore();
            LoyaltyDropdown.prototype.getContentTemplate.restore();
        });

        it('should render offers to loyalty dropdown', function() {
            var template = window.document.documentElement.querySelector('#sparksDropdownContentTemplate'),
                offersData = {
                    customerSparks: '100',
                    allActiveOffers: [],
                    availableSlottedOffers: [],
                    availableBonusOffers: []
                };

            mockGetContentTemplate.returns(template);
            getDropdownInstance(dropdownContractorOptions).renderOffersHtml(offersData);

            expect(template.querySelector('.navigation-sparks-dropdown__user').innerHTML).to.be.equal('SparksUser');
            expect(LoyaltyDropdown.prototype.addActiveOffersToTemplate.calledWith(template, offersData.allActiveOffers)).to.be.ok;
            expect(LoyaltyDropdown.prototype.addAvailableSlottedOffers.calledWith(template, offersData.availableSlottedOffers)).to.be.ok;
            expect(LoyaltyDropdown.prototype.addAvailableBonusOffers.calledWith(template, offersData.availableBonusOffers)).to.be.ok;
        });
    });

    describe('getContentTemplate method', function() {
        beforeEach(function() {
            var script = document.createElement('script');
            script.setAttribute('id', 'sparksDropdownContentTemplate');
            script.setAttribute('type', 'text/plain');
            script.innerHTML = '<div>This is a test</div>';
            document.body.appendChild(script);
        });

        afterEach(function(){
            document.getElementById('sparksDropdownContentTemplate').remove();
        });

        it('should return content template from script tag', function() {
            expect(getDropdownInstance().getContentTemplate().outerHTML).to.be.equal('<div>This is a test</div>');
        });
    });

    describe('addActiveOffersToTemplate method', function() {
        it('should hide offers list html from template if offers are not available', function() {
            getDropdownInstance().addActiveOffersToTemplate(loyaltyDOM, []);
            expect(loyaltyDOM.querySelector('.navigation-sparks-dropdown__active-offers').style.display).to.be.equal('none');
        });

        it('should add offers list to the template', function() {
            var offers = [{
                offer: {
                    offerName: 'Offer1',
                    offerId: 1234,
                    imageUrl: 'IMAGE_URL1',
                    buyUrl: 'URL1',
                    redeemed: false,
                    applicableOnline: 'applicableOnline',
                    applicableInstore: 'applicableInstore'
                }
            }, {
                offer: {
                    offerName: 'Offer2',
                    offerId: 1235,
                    imageUrl: 'IMAGE_URL2',
                    redeemed: true,
                    applicableOnline: 'applicableOnline',
                    applicableInstore: 'applicableInstore'
                }
            }, {
                offer: {
                    offerName: 'Offer3',
                    imageUrl: 'IMAGE_URL3',
                    redeemed: true,
                    applicableOnline: 'applicableOnline',
                    applicableInstore: 'applicableInstore'
                }
            }];

            getDropdownInstance().addActiveOffersToTemplate(loyaltyDOM, offers);

            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers li')[0].textContent).to.be.equal(offers[0].offer.offerName);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers li')[1].textContent).to.be.equal(offers[1].offer.offerName);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers li')[2].textContent).to.be.equal(offers[2].offer.offerName);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers img')[0].getAttribute('src')).to.be.equal(offers[0].offer.imageUrl);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers img')[1].getAttribute('src')).to.be.equal(offers[1].offer.imageUrl);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers img')[2].getAttribute('src')).to.be.equal(offers[2].offer.imageUrl);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers a')[0].getAttribute('href')).to.be.equal(offers[0].offer.buyUrl);
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers a')[1].getAttribute('href')).to.be.equal('{{globalHeader.sparksHubLink}}#');
            expect(loyaltyDOM.querySelectorAll('.navigation-sparks-dropdown__active-offers a')[2].getAttribute('href')).to.be.equal('{{globalHeader.sparksHubLink}}#');
        });
    });

    describe('addAvailableSlottedOffers method', function() {
        it('should hide available slotted offers html if offers are Zero', function() {
            getDropdownInstance().addAvailableSlottedOffers(loyaltyDOM, []);

            expect(loyaltyDOM.querySelector('.navigation-sparks-dropdown__avl-offers').style.display).to.be.equal('none');
        });

        it('should add available slotted offers html if offers are more than Zero', function() {
            var offers = [{}, {}];

            getDropdownInstance().addAvailableSlottedOffers(loyaltyDOM, offers);

            expect(loyaltyDOM.querySelector('.navigation-sparks-dropdown__avl-offers .navigation-sparks-dropdown__notification').textContent).to.be.equal(offers.length + '');
        });
    });

    describe('addAvailableBonusOffers method', function() {
        it('should hide available slotted offers html if offers are Zero', function() {
            getDropdownInstance().addAvailableBonusOffers(loyaltyDOM, []);

            expect(loyaltyDOM.querySelector('.navigation-sparks-dropdown__avl-bonus-offers').style.display).to.be.equal('none');
        });

        it('should add available slotted offers html if offers are more than Zero', function() {
            var offers = [{}, {}];

            getDropdownInstance().addAvailableBonusOffers(loyaltyDOM, offers);

            expect(loyaltyDOM.querySelector('.navigation-sparks-dropdown__avl-bonus-offers .navigation-sparks-dropdown__notification').textContent).to.be.equal(offers.length + '');
        });
    });
});

function getDropdownInstance(options) {
    return new LoyaltyDropdown(loyaltyDOM.querySelector('li'), options);
}