import loyaltyService from '../../js/loyalty/Request';
import mnsCookieService from '../../js/links/cookieService';

describe('Progressive Header Request service', function() {

    var promise,
        mockedLoyaltyOfferApi = 'http://blah.com',
        mockedLoyaltyAuthTokenUrl = 'http://www.blah.com',
        mockedAuthToken = 'AUTH_TOKEN',
        options = {
            url: mockedLoyaltyOfferApi,
            method: 'GET',
            headers: {
                'Authorization': 'MNSAuthToken ' + mockedAuthToken,
                'Accept': 'application/vnd.mns.ls-v3+json, application/json'
            }
        },
        tokenResponse = {
            status: 200,
            responseText: {
                token: mockedAuthToken
            }
        },
        offerResponse = {
            status: 200,
            responseText: 'mock response text'
        };

    beforeEach(function() {
        sinon.stub(loyaltyService, 'request');
        sinon.stub(mnsCookieService, 'getAuthTokenCookieValue');
    });

    afterEach(function() {
        loyaltyService.request.restore();
        mnsCookieService.getAuthTokenCookieValue.restore();
    });

    describe('getOffers method', function() {
        it('should return promise to be resolved with offers', function() {
            loyaltyService.request.returns(Promise.resolve(offerResponse));
            promise = loyaltyService.getOffers(mockedLoyaltyOfferApi, mockedAuthToken);

            expect(promise).to.be.instanceof(Promise);
            expect(loyaltyService.request.getCall(0).args[0]).to.deep.equal(options);
            expect(promise).to.eventually.equal('mock response text');
        });
    });

    describe('getAuthToken method', function() {
        beforeEach(function() {
            sinon.stub(loyaltyService, 'getAuthTokenFromServer');
        });

        afterEach(function() {
            loyaltyService.getAuthTokenFromServer.restore();
        });

        it('should return resolved promise if auth token cookie is available', function() {
            mnsCookieService.getAuthTokenCookieValue.returns(mockedAuthToken);

            return loyaltyService.getAuthToken(mockedLoyaltyAuthTokenUrl).then(function(authToken) {
                return expect(authToken).to.be.equal(mockedAuthToken);
            });
        });

        it('should call getAuthTokenFromServer method if auth token cookie not available', function() {
            mnsCookieService.getAuthTokenCookieValue.returns(null);

            loyaltyService.getAuthToken(mockedLoyaltyAuthTokenUrl);

            return expect(loyaltyService.getAuthTokenFromServer.calledOnce).to.be.ok;
        });
    });

    describe('getAuthTokenFromServer method', function() {

        it('should return resolved promise if auth token available', function() {
            loyaltyService.request.returns(Promise.resolve(tokenResponse));

            return expect(loyaltyService.getAuthTokenFromServer(mockedLoyaltyAuthTokenUrl)).to.eventually.equal(mockedAuthToken);
        });

        it('should return rejected promise if auth token not available', function() {
            loyaltyService.request.returns(Promise.reject(new Error('error')));

            return loyaltyService.getAuthTokenFromServer(mockedLoyaltyAuthTokenUrl).catch(function(error) {
                expect(error).to.be.instanceof(Error);
            });
        });
    });
});