import MiniBagOverlay from '../../js/miniBag/Index';
import DockOverlay from '../../js/dockOverlay/Index';
import helper from './helper/helper';
import miniBagService from '../../js/miniBag/service';
import analytics from '../../js/analytics';

describe ('Progressive Header MiniBagOverlay Component', function() {

    var overlayDOM,
        targetDOM,
        miniBagTemplate;

    beforeEach(function(){
        overlayDOM = helper.getOverlayDOM();
        targetDOM = helper.getOverlayTarget();

        document.body.appendChild(overlayDOM);
        document.body.appendChild(targetDOM);
    });

    afterEach(function(){
        overlayDOM.remove();
        targetDOM.remove();
    });


    describe('constructor method', function() {
        it('should return MiniBagOverlay instance', function() {
            var overlay = new MiniBagOverlay(overlayDOM);

            expect(overlay).to.be.instanceof(MiniBagOverlay);
            expect(overlay.overlay).to.be.instanceof(DockOverlay);
        });
    });

    describe('initialize method', function() {
        it('should return MiniBagOverlay instance list', function() {
            var list = MiniBagOverlay.initialize([overlayDOM.cloneNode(true), overlayDOM.cloneNode(true), overlayDOM.cloneNode(true)]);

            expect(list.length).to.be.equal(3);
            expect(list[0]).to.be.instanceof(MiniBagOverlay);
            expect(list[1]).to.be.instanceof(MiniBagOverlay);
            expect(list[2]).to.be.instanceof(MiniBagOverlay);
        });
    });

    describe('getOptions method', function() {
        it('should return mini bag overlay options', function() {
            var overlay = new MiniBagOverlay(overlayDOM.cloneNode(true)),
                options = {
                    targetSelector: overlayDOM.getAttribute('data-mini-bag-link'),
                    align: overlayDOM.getAttribute('data-mini-bag-align'),
                    direction: overlayDOM.getAttribute('data-mini-bag-direction')
                };

            expect(overlay.getOptions()).to.deep.equal(options);
        });
    });

    describe('bindEvents method', function() {
        var miniBagOverlay,
            mockedBasketSize;

        beforeEach(function() {
            miniBagOverlay = new MiniBagOverlay(overlayDOM.cloneNode(true));
        });

        describe('data event handlers', function() {

            beforeEach(function() {
                mockedBasketSize = sinon.stub(miniBagOverlay, 'getBasketSize');
                miniBagOverlay.removeOnInitShowEventHandler = null;
                sinon.stub(miniBagOverlay.overlay, 'onInitShowEvent').returns(function(){});
            });

            afterEach(function() {
                miniBagOverlay.overlay.onInitShowEvent.restore();
                mockedBasketSize.restore();
            });

            it('should add load data event handler if basket size is greater than 0', function() {
                mockedBasketSize.returns(3);
                miniBagOverlay.bindEvents();

                expect(miniBagOverlay.overlay.onInitShowEvent.calledOnce).to.be.ok;
                expect(miniBagOverlay.removeOnInitShowEventHandler).to.not.be.null;
            });

            it('should not add load data event handler if basket size is 0', function() {
                mockedBasketSize.returns(0);
                miniBagOverlay.bindEvents();

                expect(miniBagOverlay.overlay.onInitShowEvent.called).to.not.be.ok;
                expect(miniBagOverlay.removeOnInitShowEventHandler).to.be.null;
            });
        });


        it('should call miniBagService.onChange method', function() {
            sinon.spy(miniBagService, 'onChange');
            miniBagOverlay.bindEvents();
            expect(miniBagService.onChange.calledOnce).to.be.ok;
            miniBagService.onChange.restore();
        });
    });

    describe('loadData method', function() {
        var spy;

        beforeEach(function(){
            spy = sinon.spy(miniBagService, 'load');
        });

        afterEach(function(){
            miniBagService.load.restore();
        });

        it('should call mini bag service to load data', function() {
            var overlay = new MiniBagOverlay(overlayDOM, {miniBagCMD : 'http://test'});

            overlay.removeOnInitShowEventHandler = sinon.stub();

            overlay.loadData();

            expect(overlay.removeOnInitShowEventHandler.calledOnce).to.be.ok;
            expect(spy.calledOnce).to.be.ok;
        });
    });

    describe('basketUpdated method', function() {
        var miniBagOverlay,
            spyDisableOverlay,
            spyEnableOverlay,
            stubGenerateMiniBagContent;

        beforeEach(function() {
            miniBagOverlay = new MiniBagOverlay(overlayDOM);
            spyDisableOverlay = sinon.spy(miniBagOverlay.overlay, 'disable');
            spyEnableOverlay = sinon.spy(miniBagOverlay.overlay, 'enable');
            stubGenerateMiniBagContent = sinon.stub(miniBagOverlay, 'generateMiniBagContent');
            sinon.stub(miniBagOverlay, 'insertMiniBag');
            sinon.stub(miniBagOverlay, 'insertMiniBagCarousel');
            sinon.stub(miniBagOverlay, 'disableScroll');
            sinon.stub(miniBagOverlay, 'initMiniBagCarousel');
        });

        afterEach(function() {
            spyDisableOverlay.restore();
            spyEnableOverlay.restore();
            miniBagOverlay.insertMiniBag.restore();
            miniBagOverlay.insertMiniBagCarousel.restore();
            miniBagOverlay.disableScroll.restore();
            miniBagOverlay.initMiniBagCarousel.restore();
        });

        it('should disable mini bag overlay if basket has 0 items', function() {
            miniBagOverlay.basketUpdated({
                basketDetails: []
            });
            expect(spyDisableOverlay.calledOnce).to.be.ok;
        });

        it('should add mini bag content if basket has less than equals to 2 items', function() {
            var data = {basketDetails: [{childItemCount:0}, {childItemCount:0}]};

            stubGenerateMiniBagContent.returns('<div>Mini Bag Content</div>');
            miniBagOverlay.basketUpdated(data);

            expect(spyEnableOverlay.calledOnce).to.be.ok;
            expect(miniBagOverlay.generateMiniBagContent.calledWith(data)).to.be.ok;
            expect(miniBagOverlay.insertMiniBag.calledWith('<div>Mini Bag Content</div>')).to.be.ok;
            expect(miniBagOverlay.disableScroll.calledWith(true)).to.be.ok;
        });

        it('should add mini bag content if basket has greater than 2 items', function() {
            var data = {basketDetails: [{}, {}, {}, {}, {}]};

            stubGenerateMiniBagContent.returns('<div>Mini Bag Carousel</div>');
            miniBagOverlay.basketUpdated(data);

            expect(spyEnableOverlay.calledOnce).to.be.ok;
            expect(miniBagOverlay.generateMiniBagContent.calledWith(data)).to.be.ok;
            expect(miniBagOverlay.insertMiniBagCarousel.calledWith('<div>Mini Bag Carousel</div>')).to.be.ok;
            expect(miniBagOverlay.disableScroll.calledWith(false)).to.be.ok;
            expect(miniBagOverlay.initMiniBagCarousel.calledOnce).to.be.ok;
        });
    });

    describe('insertMiniBag method', function() {
        it('should insert mini bag dom into overlay', function() {
            var miniBagOverlay = new MiniBagOverlay(overlayDOM),
                content = document.createElement('div');

            content.innerHTML = '<div>Mini bag content</div>';
            miniBagOverlay.overlay.content = sinon.stub();
            miniBagOverlay.insertMiniBag(content);

            expect(miniBagOverlay.overlay.content.calledOnce).to.be.ok;
        });
    });

    describe('insertMiniBagCarousel method', function() {
        it('should insert mini bag carousel dom into overlay', function() {
            var miniBagOverlay = new MiniBagOverlay(overlayDOM),
                content = document.createElement('div');

            content.innerHTML = '<div>Mini bag title</div><div>Mini bag content</div>';
            miniBagOverlay.overlay.content = sinon.stub();
            miniBagOverlay.insertMiniBagCarousel(content);

            expect(miniBagOverlay.overlay.content.calledOnce).to.be.ok;
        });
    });

    describe('getBasketSize method', function() {
        it('should return "0" from cookie value if basket count not exists', function() {
            var miniBagOverlay = new MiniBagOverlay(overlayDOM);

            expect(miniBagOverlay.getBasketSize()).to.be.equal('0');
        });

        it('should return "NUMBER" from cookie value if basket count exists', function() {
            var miniBagOverlay = new MiniBagOverlay(overlayDOM);

            document.cookie = 'basketSize_12345=5';

            expect(miniBagOverlay.getBasketSize()).to.be.equal('5');
        });
    });

    describe('getMiniBagTemplate method', function() {
        beforeEach(function() {
            miniBagTemplate = document.createElement('script');

            miniBagTemplate.setAttribute('id', 'hdr-miniBagTemplate');
            miniBagTemplate.setAttribute('type', 'text/plain');
            miniBagTemplate.innerHTML = '<div>mini basket</div>';

            document.body.appendChild(miniBagTemplate);
        });

        afterEach(function() {
            miniBagTemplate.remove();
        });

        it('should return mini bag DOM from script template #hdr-miniBagTemplate', function() {
            var miniBagOverlay = new MiniBagOverlay(overlayDOM);

            expect(miniBagOverlay.getMiniBagTemplate().outerHTML).to.equal('<div>mini basket</div>');
        });
    });

});
