import XSmallDevice from '../../js/navigation/XSmallDevice';
import util from '../../js/util';

describe('x-small device', () => {

    let mockedLevel2 = {},
        mockedLevel3 = {},
        sandbox,
        xSmallDevice;

    beforeEach(() => {
        sandbox = sinon.sandbox.create();
        mockedLevel2 = {
            activate: sandbox.spy(),
            deactivate: sandbox.spy(),
            navPrimaryAList: []
        };
        mockedLevel3 = {
            navPrimarySubmenuList: []
        };

        document.body.appendChild(getTouchDeviceDOM());
        xSmallDevice = new XSmallDevice(mockedLevel2, mockedLevel3);
    });

    afterEach(() => {
        sandbox.restore();
    });

    describe('constructor method', () => {

        it('adds a level2 onto the x small device instance', () => {
            expect(xSmallDevice.level2).to.equal(mockedLevel2);
        });

        it('sets the body element onto the x small device instance', () => {
            expect(xSmallDevice.bodyEl).to.equal(document.querySelector('body'));
        });

        it('sets the primary nav section element onto the x small device instance', () => {
            expect(xSmallDevice.navPrimarySectionEl).to.equal(document.querySelector('.navigation-primary__section'));
        });

        it('sets the fixed to pmenu element onto the x small device instance', () => {
            expect(xSmallDevice.navPrimaryFixedTopBar).to.equal(document.querySelector('.navigation-burger__fixedtopmenu'));
        });

        it('sets the navigation overlay element onto the x small device instance', () => {
            expect(xSmallDevice.navigationOverlayEl).to.equal(document.querySelector('#navigationOverlay'));
        });

        it('sets the hamburger element onto the x small device instance', () => {
            expect(xSmallDevice.hamburgerEl).to.equal(document.querySelector('.navigation-res-nav-header__item--menu'));
        });

        it('sets the back label element onto the x small device instance', () => {
            expect(xSmallDevice.backLabelEl).to.equal(document.querySelector('.navigation-submenu__back-bar-label'));
        });

        it('sets the back link element onto the x small device instance', () => {
            expect(xSmallDevice.backButtonEl).to.equal(document.querySelector('.navigation-submenu__back-bar-link'));
        });

        it('sets the level2 headers elements onto the x small device instance', () => {
            expect(xSmallDevice.level2Headers).to.equal(document.querySelectorAll('#navContainer .nav-submenu__l2-header'));
        });

        it('sets the header section element onto the x small device instance', () => {
            expect(xSmallDevice.headerEl).to.equal(document.querySelector('#progressiveHeaderSection'));
        });

        it('sets the interaction event to be a click event', () => {
            expect(xSmallDevice.interactionEvent).to.equal('click');
        });

        it('sets the interaction event to be the touch start event when has touch capability enabled', () => {
            sandbox.stub(util, 'hasTouchCapability').returns(true);
            xSmallDevice = new XSmallDevice(mockedLevel2, mockedLevel3);
            expect(xSmallDevice.interactionEvent).to.equal('touchstart');
        });
    });

    describe('setup method', () => {
        beforeEach(() => {
            sandbox.stub(xSmallDevice, 'myAccount');
            sandbox.stub(xSmallDevice, 'backClick');
            sandbox.stub(xSmallDevice, 'addAnimation');
            xSmallDevice.setup();
        });

        it('calls function for displaying the my account details', () => {
            expect(xSmallDevice.myAccount).to.have.been.calledOnce;
        });

        it('makes sure the navigation level2 overlay is closed', () => {
            xSmallDevice.navContainerEl.classList.add('navigation-is-active');
            xSmallDevice.setup();
            expect(xSmallDevice.backClick).to.have.been.called;
        });
    });

    describe('teardown method', () => {
        beforeEach(() => {
            sandbox.stub(xSmallDevice, 'removeAnimation');
            sandbox.stub(xSmallDevice, 'closeMenu');
            xSmallDevice.teardown();
        });

        it('deactivates the level 2 nagiation object', () => {
            expect(mockedLevel2.deactivate).to.have.been.calledOnce;
        });

        it('removes animation', () => {
            expect(xSmallDevice.removeAnimation).to.have.been.calledOnce;
        });

        it('closes the menu', () => {
            expect(xSmallDevice.closeMenu).to.have.been.calledOnce;
        });

    });

    describe('add animation method', () => {	
        beforeEach(() => {	
            xSmallDevice.addAnimation();	
        });	

         it('animates the primary nav section', () => {	
            expect(xSmallDevice.navPrimarySectionEl.style.transition).to.equal('transform .1s cubic-bezier(0, 0.3, 0.2, 1.02) 0s');
            expect(xSmallDevice.navPrimaryFixedTopBar.style.transition).to.equal('transform .1s cubic-bezier(0, 0.3, 0.2, 1.02) 0s');
        });	
    });	

    describe('removes animation method', () => {
        beforeEach(() => {
            xSmallDevice.removeAnimation();
        });

        it('removes the primary nav section animation', () => {
            expect(xSmallDevice.navPrimarySectionEl.style.transition).to.equal('none');
            expect(xSmallDevice.navPrimaryFixedTopBar.style.transition).to.equal('none');
        });
    });

    describe('myAccount method', () => {
        it('removes the primary nav section animation', () => {
            expect(typeof xSmallDevice.myAccount).to.equal('function');
        });
    });

    describe('displayNode method', () => {
        it('shows an element', () => {
            let mockeElement = document.createElement('div');

            xSmallDevice.displayNode(mockeElement, true);
            expect(mockeElement.style.display).to.equal('block');
        });

        it('hides an element', () => {
            let mockeElement = document.createElement('div');

            xSmallDevice.displayNode(mockeElement, false);
            expect(mockeElement.style.display).to.equal('none');
        });
    });

    describe('level2Click method', () => {
        it('prevents the default event action', () => {
            let mockedEvent = {
                preventDefault: sinon.spy(),
                target: document.createElement('div')
            };

            xSmallDevice.level2Click(mockedEvent);
            expect(mockedEvent.preventDefault).to.have.been.called;
        });

        it('activates the level 2 object if not already active', () => {
            let mockedEvent = {
                preventDefault: sinon.spy(),
                target: document.createElement('div')
            };

            xSmallDevice.level2Click(mockedEvent);
            expect(mockedLevel2.activate).to.have.been.calledWith(mockedEvent.target);

        });

        it('set the back label to tht of the click element if not already active', () => {
            let mockedEvent,
                mockedLabel = "some label",
                mockedElement = document.createElement('div'),
                mockedPath = 'c/women',
                mockedElementURL = document.createElement('a');

            mockedElement.innerHTML = mockedLabel;
            mockedElementURL.pathname = mockedPath;
            mockedEvent = {
                preventDefault: sinon.spy(),
                target: mockedElement,
                pathname: mockedElementURL.mockedPath
            };
            sandbox.stub(xSmallDevice, 'setBackLabel');
            xSmallDevice.level2Click(mockedEvent);
            expect(xSmallDevice.setBackLabel).to.have.been.calledWith(mockedLabel, mockedElementURL.mockedPath);
        });

        it('set the back label to tht of the click element if not already active', () => {
            let mockedEvent,
                mockedLabel = "some label",
                mockedPath = "/c/women",
                mockedElementURL = document.createElement('a'),
                mockedElement = document.createElement('div');

            mockedElement.innerHTML = mockedLabel;
            mockedElementURL.pathname = mockedPath;
            mockedEvent = {
                preventDefault: sinon.spy(),
                target: mockedElement,
                pathname: mockedElementURL.mockedPath
            };
            sandbox.stub(xSmallDevice, 'setBackLabel');
            xSmallDevice.level2Click(mockedEvent);
            expect(xSmallDevice.setBackLabel).to.have.been.calledWith(mockedLabel, mockedElementURL.mockedPath);
        });
    });

    describe('mobileMenuToggle method', () => {
        let mockedEvent = {
            preventDefault: sinon.spy()
        };

        beforeEach(() => {
            sandbox.stub(xSmallDevice, 'openMenu');
            sandbox.stub(xSmallDevice, 'closeMenu');
        });

        it('closes the menu', () => {
            xSmallDevice.navPrimarySectionEl.classList.add('navigation-is-active');
            xSmallDevice.mobileMenuToggle(mockedEvent);
            expect(xSmallDevice.closeMenu).to.have.been.called;
        });

        it('opens the menu', () => {
            xSmallDevice.navPrimarySectionEl.className = '';
            xSmallDevice.mobileMenuToggle(mockedEvent);
            expect(xSmallDevice.openMenu).to.have.been.called;
        });
    });

    describe('openMenu method', () => {

    });

    describe('closeMenu method', () => {

    });

    function getTouchDeviceDOM() {
        let domEl = document.createElement('div'),
            html = `<header class="Header">
                        <section></section>
                        <div id="navContainer">
                            <div class="navigation-burger__fixedtopmenu"></div>
                            <div class="navigation-primary__section"></div>
                            <div id="navigationOverlay"></div>
                            <div class="isUserSparksMember"></div>
                            <div class="nav-primary__menu-link"></div>
                            <div class="navigation-is-active"></div>
                            <div class="navigation-submenu__back-bar-label"></div>
                            <div class="navigation-submenu__back-bar-link"></div>
                            <div class="nav-submenu__l2-header"></div>
                        </div>
                    </header>`;

        domEl.innerHTML = html;
        return domEl;
    }

});
