import savedItemService from '../../js/savedItems/savedItemService';
import cookieService from '../../js/links/cookieService';

describe('Progressive Header savedItemService Component', function() {
    beforeEach(function() {
        sinon.stub(cookieService, 'getSavedItemCountCookieValue');
        savedItemService.itemCount = null;
    });
    afterEach(function() {
        cookieService.getSavedItemCountCookieValue.restore();
    });

    describe('itemCount setter/getter property', function() {
        it('should set initial itemCount value from cookieService', function() {
            cookieService.getSavedItemCountCookieValue.returns(4);
            expect(savedItemService.itemCount).to.equal(4);
        });

        it('should set and get itemCount value', function() {
            savedItemService.itemCount = 5;
            expect(savedItemService.itemCount).to.equal(5);
        });
    });

    describe('onChange method', function() {
        it('should add change listener callback if already not exists', function() {
            var func = sinon.stub();

            savedItemService.onChange(func);
            savedItemService.update([]);
            expect(func.calledOnce).to.be.ok;
        });

        it('should not add change listener callback if already exists', function() {
            var func = sinon.stub();

            savedItemService.onChange(func);
            savedItemService.onChange(func);
            savedItemService.update([]);
            expect(func.calledOnce).to.be.ok;
        });
    });

    describe('update method', function() {
        it('should update saved item data', function() {
            var data = [{}, {}, {}];

            savedItemService.update(data);
            expect(savedItemService.getData()).to.equal(data);
        });
    });

    describe('getData method', function() {
        it('should returns saved item data', function() {
            var data = [{}, {}, {}];

            savedItemService.update(data);
            expect(savedItemService.getData()).to.equal(data);
        });
    });

    describe('getItemById method', function() {
        it('should returns saved item by orderItemId', function() {
            var data = [{orderItemId: 1}, {orderItemId: 2}, {orderItemId: 3}];

            savedItemService.update(data);
            expect(savedItemService.getItemById(3)).to.equal(data[2]);
        });
    });

    describe('toQueryString method', function() {
        it('should return a string concatenated list of params from object', function() {
            var params = {
                storeId: 1234,
                'langId': '-24',
                catalogId: '10051',
                resJSON:true
            };

            var stringParams = savedItemService.toQueryString(params);
            expect(stringParams).to.equal('storeId=1234&langId=-24&catalogId=10051&resJSON=true');
        });

        it('should handle invalid params - string', function() {
            var params = 'storeId=1234&langId=-24';

            var stringParams = savedItemService.toQueryString(params);
            expect(stringParams).to.equal('');
        });

        it('should handle invalid params - number', function() {
            var params = 1234;

            var stringParams = savedItemService.toQueryString(params);
            expect(stringParams).to.equal('');
        });

        it('should handle invalid params - function', function() {
            var params = function () {
                return '';
            };

            var stringParams = savedItemService.toQueryString(params);
            expect(stringParams).to.equal('');
        });

        it('should handle invalid params - empty object', function() {
            var stringParams = savedItemService.toQueryString({});
            expect(stringParams).to.equal('');
        });

        it('should handle no params', function() {
            var stringParams = savedItemService.toQueryString();
            expect(stringParams).to.equal('');
        });
    });

    describe('loadData, addToBag and remove method', function() {
        var existingData = [{addBagLink: {action: 'link1'}, removeLink: {action: 'link1'}},
            {addBagLink: {action: 'link2'}, removeLink: {action: 'link2'}}],
        thisData = {addBagLink: {action: 'link2'}, removeLink: {action: 'link2'}},
        url = 'http://blah.com',
        newData = [{addBagLink: {action: 'link1'}, removeLink: {action: 'link1'}}];

        beforeEach(function () {
            sinon.stub(savedItemService, 'request');
            savedItemService.update(existingData);
            savedItemService.request.returns(Promise.resolve({status:200, responseText: existingData}));
        });
        afterEach(function () {
            savedItemService.request.restore();
        });
        it('should remove the added item from saved item', function() {
            expect(savedItemService.getData()).to.deep.equal(existingData);

            return savedItemService.addToBag(thisData, url).then(function(){
                return expect(savedItemService.getData()).to.deep.equal(newData);
            });
        });

        it('should remove item from saved item', function() {
            expect(savedItemService.getData()).to.deep.equal(existingData);

            return savedItemService.remove(thisData, url).then(function(){
                return expect(savedItemService.getData()).to.deep.equal(newData);
            });
        });

        it('should load data as saved item', function() {

            return savedItemService.loadData('foo', 'bar').then(function(){
                return expect(savedItemService.getData()).to.deep.equal(existingData);
            });
        });
    });
});
