import util from '../js//util';

describe('header utilities module', () => {
    it('should flag if a browser has touch capability', () => {
        document.documentElement.className = 'something touch';
        expect(util.hasTouchCapability()).to.equal(true);
    });

    it('should prevent event propagation and prevent default event behaviour for a given click event', () => {
        let mockedEvent = {
            preventDefault: () => {},
            stopPropagation: () => {}
        };

        sinon.spy(mockedEvent, 'preventDefault');
        sinon.spy(mockedEvent, 'stopPropagation');
        util.preventClick(mockedEvent);

        expect(mockedEvent.preventDefault).to.have.been.called;
        expect(mockedEvent.stopPropagation).to.have.been.called;
    });

    it('should add a class to an element if its NOT already present', () => {
        let mockedEl = document.createElement('div');

        mockedEl.className = 'something';
        util.addClass(mockedEl, 'something-else');

        expect(mockedEl.className).to.equal('something something-else');
    });

    it('should NOT add a class to an element if it IS already present', () => {
        let mockedEl = document.createElement('div');

        mockedEl.className = 'something';
        util.addClass(mockedEl, 'something');

        expect(mockedEl.className).to.equal('something');
    });

    it('should remove a class from an element if it is already present', () => {
        let mockedEl = document.createElement('div');

        mockedEl.className = 'something something-two something-three';
        util.removeClass(mockedEl, 'something-two');

        expect(mockedEl.className).to.equal('something something-three');
    });

    it('should NOT remove a class from an element if it is NOT already present', () => {
        let mockedEl = document.createElement('div');

        mockedEl.className = 'something something-two';
        util.removeClass(mockedEl, 'something-three');

        expect(mockedEl.className).to.equal('something something-two');
    });

    it('should return true if a class is present on an element', () => {
        let mockedEl = document.createElement('div');

        mockedEl.className = 'something something-two something-three';
        var result = util.hasClass(mockedEl, 'something-two');

        expect(result).to.equal(true);
    });

    it('should return false if a class is NOT present on an element', () => {
        let mockedEl = document.createElement('div');

        mockedEl.className = 'something something-two something-three';
        var result = util.hasClass(mockedEl, 'something-four');

        expect(result).to.equal(false);
    });

    it('should serialize an object into a query string', () => {

        let dataObj = {
            param1: 'value1',
            param2: 'value2'
        };

        expect(util.serialize(dataObj)).to.equal('param1=value1&param2=value2');
        expect(util.serialize({})).to.equal('');
        expect(util.serialize(['test'])).to.equal(false);
        expect(util.serialize()).to.equal(false);
        expect(util.serialize('')).to.equal(false);
        expect(util.serialize([])).to.equal(false);
    });

    it('should construct a url from host and params object', () => {
        expect(util.constructUrl('test.com', {
            param1 : 'value1',
            param2 : 'value2'
        })).to.equal('test.com?param1=value1&param2=value2');

        expect(util.constructUrl({})).to.equal('');

        expect(util.constructUrl('test.com')).to.equal('test.com');

        expect(util.constructUrl('test.com', 'value1')).to.equal('test.com');

        expect(util.constructUrl('test.com', {})).to.equal('test.com');
    });

    describe('appendSourceUrl method', function() {
        var sourceUrl = 'http://localhost:3000/example/mns-core-ui';

        it('should append the source url', function() {
            var newTarget = util.appendSourceUrl('http://localhost:3000/regForm?catalogId=10051', sourceUrl);
            expect(newTarget).to.be.equal('http://localhost:3000/regForm?catalogId=10051&sourceurl=http%3A%2F%2Flocalhost%3A3000%2Fexample%2Fmns-core-ui');
        });

        it('should append the source url if no other parameters', function() {
            var newTarget = util.appendSourceUrl('http://localhost:3000/regForm', sourceUrl);
            expect(newTarget).to.be.equal('http://localhost:3000/regForm?sourceurl=http%3A%2F%2Flocalhost%3A3000%2Fexample%2Fmns-core-ui');
        });

        it('should return to root if invalid arguments', function() {
            var newTarget = util.appendSourceUrl(1234, 1234);
            expect(newTarget).to.be.equal('/');
        });

        it('should not append question mark if target ends in question mark', function() {
           var newTarget = util.appendSourceUrl('http://localhost:3000/regForm?', sourceUrl);
           expect(newTarget).to.be.equal('http://localhost:3000/regForm?sourceurl=http%3A%2F%2Flocalhost%3A3000%2Fexample%2Fmns-core-ui');
        });
    });

    describe('setTabIndex', function() {
        var DOMElements;

        beforeEach(function() {
            DOMElements = getDOMElements().getElementsByTagName('a');
        });

        it('should set the elements tabindex to 0' , function() {
            util.setTabIndex(DOMElements, 0);
            expect(DOMElements[0].getAttribute('tabindex')).to.be.equal('0');
        });

        it('should set the elements tabindex to 1' , function() {
            util.setTabIndex(DOMElements, 1);
            expect(DOMElements[0].getAttribute('tabindex')).to.be.equal('1');
        });
    });

    function getDOMElements() {
        var domEl = document.createElement('div');
        domEl.innerHTML = '<a href="#" tabindex="-1">Link 1</a><a href="#" tabindex="-1">Link 2</a><a href="#" tabindex="-1">Link 3</a>';
        return domEl;
    }

});
